/*
  positive_min.hpp

  Copyright (C) 2016 Taiji Yamada <taiji@aihara.co.jp>. Distributed
  under the Boost Software License, Version 1.0. (See accompanying file
  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
*/
#ifndef BOOST_ACCUMULATORS_STATISTICS_POSITIVE_MIN_HPP
#define BOOST_ACCUMULATORS_STATISTICS_POSITIVE_MIN_HPP

#include <limits>
#include <boost/mpl/placeholders.hpp>
#include <boost/accumulators/framework/accumulator_base.hpp>
#include <boost/accumulators/framework/extractor.hpp>
#include <boost/accumulators/framework/parameters/sample.hpp>
#include <boost/accumulators/numeric/functional.hpp>
#include <boost/accumulators/framework/depends_on.hpp>
#include <boost/accumulators/statistics_fwd.hpp>

namespace boost { namespace accumulators
{

namespace impl
{
    ///////////////////////////////////////////////////////////////////////////////
    // positive_min_impl
    template<typename Sample>
    struct positive_min_impl
      : accumulator_base
    {
        // for boost::result_of
        typedef Sample result_type;

        template<typename Args>
        positive_min_impl(Args const &args)
          : dirty(true)
        {
	    if (result_type(0) < numeric::as_max(args[sample | Sample()])) {
	        positive_min_ = numeric::as_max(args[sample | Sample()]);
		dirty = false;
	    }
        }

        template<typename Args>
        void operator ()(Args const &args)
        {
	    if (result_type(0) < args[sample]) {
	        if (dirty) {
		    this->positive_min_ = args[sample];
		    dirty = false;
		}
		else
		    numeric::min_assign(this->positive_min_, args[sample]);
	    }
        }

        result_type result(dont_care) const
        {
            return this->positive_min_;
        }

    private:
        bool dirty;
        Sample positive_min_;
    };

} // namespace impl

///////////////////////////////////////////////////////////////////////////////
// tag::positive_min
//
namespace tag
{
    struct positive_min
      : depends_on<>
    {
        /// INTERNAL ONLY
        ///
        typedef accumulators::impl::positive_min_impl<mpl::_1> impl;
    };
}

///////////////////////////////////////////////////////////////////////////////
// extract::positive_min
//
namespace extract
{
    extractor<tag::positive_min> const positive_min = {};

    BOOST_ACCUMULATORS_IGNORE_GLOBAL(positive_min)
}

using extract::positive_min;

}}

#endif
