/*
  container_numeric_operators.cc

  Written by Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <cassert>
#include <valarray>
#include <vector>
#include "valarray_numeric.hpp"
#include "vector_numeric.hpp"
#include "valarray_numeric_quantified_operators.hpp"
#include "valarray_numeric_ordered_operators.hpp"
#include "vector_numeric_quantified_operators.hpp"
#include "vector_numeric_ordered_operators.hpp"

template <typename T, typename U>
static inline
bool operator==(const std::valarray<T> &a, const std::vector<U> &b)
{
  if (!(a.size() == b.size())) return false;
  for (size_t i=0; i<a.size(); ++i)
    if (!(a[i] == b[i] || (std::isnan(a[i]) && std::isnan(b[i]))))
      return false;
  return true;
}

int main()
{
#if !0 // not T as real
  typedef long long T;
#else
  typedef double T;
#endif
  const size_t n = 3, m = 10/2;

  for (size_t k=0; k<pow(m, n); ++k) {
    std::valarray<T> a(n);
    std::vector<T> A(n);
    size_t q = k;
    for (size_t i=0; i<n; ++i) {
      a[i] = (q % m);
      A[i] = (q % m);
      q /= m;
    }
    /*
      unary operators
    */
#if defined(_VECTOR_NUMERIC_HPP_)
    {
      std::valarray<T> c = +a;
      std::vector<T> C = +A;
      assert(c == C);
    }
    {
      std::valarray<T> c = -a;
      std::vector<T> C = -A;
      assert(c == C);
    }
#if !0 // not T as real
    {
      std::valarray<T> c = ~a;
      std::vector<T> C = ~A;
      assert(c == C);
    }
#endif
    {
      std::valarray<bool> c = !a;
      std::vector<bool> C = !A;
      assert(c == C);
    }
    /*
      unary functions
    */
#if defined(_VALARRAY_NUMERIC_HPP_)
    {
      T c = norm2(a);
      T C = norm2(A);
      assert(c == C);
    }
    {
      T c = norm(a, 0);
      T C = norm(A, 0);
      assert(c == C);
    }

    {
      T c = prod(a);
      T C = prod(A);
      assert(c == C);
    }
#endif
    {
      std::valarray<T> c = abs(a);
      std::vector<T> C = abs(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = sqrt(a);
      std::vector<T> C = sqrt(A);
      assert(c == C);
    }
#if defined(_VALARRAY_NUMERIC_HPP_)
    {
      std::valarray<T> c = cbrt(a);
      std::vector<T> C = cbrt(A);
      assert(c == C);
    }
#endif
    {
      std::valarray<T> c = exp(a);
      std::vector<T> C = exp(A);
      assert(c == C);
    }
#if defined(_VALARRAY_NUMERIC_HPP_)
    {
      std::valarray<T> c = exp2(a);
      std::vector<T> C = exp2(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = expm1(a);
      std::vector<T> C = expm1(A);
      assert(c == C);
    }
#endif
    {
      std::valarray<T> c = log(a);
      std::vector<T> C = log(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = log10(a);
      std::vector<T> C = log10(A);
      assert(c == C);
    }
#if defined(_VALARRAY_NUMERIC_HPP_)
    {
      std::valarray<T> c = log2(a);
      std::vector<T> C = log2(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = log1p(a);
      std::vector<T> C = log1p(A);
      assert(c == C);
    }
#endif
    {
      std::valarray<T> c = sin(a);
      std::vector<T> C = sin(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = cos(a);
      std::vector<T> C = cos(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = tan(a);
      std::vector<T> C = tan(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = sinh(a);
      std::vector<T> C = sinh(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = cosh(a);
      std::vector<T> C = cosh(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = tanh(a);
      std::vector<T> C = tanh(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = asin(a);
      std::vector<T> C = asin(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = acos(a);
      std::vector<T> C = acos(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = atan(a);
      std::vector<T> C = atan(A);
      assert(c == C);
    }
#if defined(_VALARRAY_NUMERIC_HPP_)
    {
      std::valarray<T> c = asinh(a);
      std::vector<T> C = asinh(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = acosh(a);
      std::vector<T> C = acosh(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = atanh(a);
      std::vector<T> C = atanh(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = erf(a);
      std::vector<T> C = erf(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = erfc(a);
      std::vector<T> C = erfc(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = tgamma(a);
      std::vector<T> C = tgamma(A);
      assert(c == C);
    }
    {
      std::valarray<T> c = lgamma(a);
      std::vector<T> C = lgamma(A);
      assert(c == C);
    }
#endif
#endif
    for (T d(0); d<T(m); ++d) {
      /*
	binary operators
      */
#if defined(_VECTOR_NUMERIC_HPP_)
      // binary compound assignment operators
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c += d;
	C += d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c -= d;
	C -= d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c *= d;
	C *= d;
	assert(c == C);
      }
      if (d != 0) {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c /= d;
	C /= d;
	assert(c == C);
      }
#if !0 // not T as real
      if (d != 0) {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c %= d;
	C %= d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c &= d;
	C &= d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c |= d;
	C |= d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c ^= d;
	C ^= d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c <<= d;
	C <<= d;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c >>= d;
	C >>= d;
	assert(c == C);
      }
#endif
      // binary operators
      {
	std::valarray<T> c(a + d);
	std::vector<T> C(A + d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a - d);
	std::vector<T> C(A - d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a * d);
	std::vector<T> C(A * d);
	assert(c == C);
      }
      if (d != 0) {
	std::valarray<T> c(a / d);
	std::vector<T> C(A / d);
	assert(c == C);
      }
#if !0 // not T as real
      if (d != 0) {
	std::valarray<T> c(a % d);
	std::vector<T> C(A % d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a & d);
	std::vector<T> C(A & d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a | d);
	std::vector<T> C(A | d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a ^ d);
	std::vector<T> C(A ^ d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a << d);
	std::vector<T> C(A << d);
	assert(c == C);
      }
      {
	std::valarray<T> c(a >> d);
	std::vector<T> C(A >> d);
	assert(c == C);
      }
#endif
      // binary operators
      {
	std::valarray<T> c(d + a);
	std::vector<T> C(d + A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d - a);
	std::vector<T> C(d - A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d * a);
	std::vector<T> C(d * A);
	assert(c == C);
      }
      if (find(A.begin(), A.end(), T(0)) == A.end()) {
	std::valarray<T> c(d / a);
	std::vector<T> C(d / A);
	assert(c == C);
      }
#if !0 // not T as real
      if (find(A.begin(), A.end(), T(0)) == A.end()) {
	std::valarray<T> c(d % a);
	std::vector<T> C(d % A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d & a);
	std::vector<T> C(d & A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d | a);
	std::vector<T> C(d | A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d ^ a);
	std::vector<T> C(d ^ A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d << a);
	std::vector<T> C(d << A);
	assert(c == C);
      }
      {
	std::valarray<T> c(d >> a);
	std::vector<T> C(d >> A);
	assert(c == C);
      }
#endif
      // binary comparison operators
      {
	std::valarray<bool> c(a < d);
	std::vector<bool> C(operator_less(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a <= d);
	std::vector<bool> C(operator_less_equal(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a > d);
	std::vector<bool> C(operator_greater(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a >= d);
	std::vector<bool> C(operator_greater_equal(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a == d);
	std::vector<bool> C(operator_equal_to(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a != d);
	std::vector<bool> C(operator_not_equal_to(A, d));
	assert(c == C);
      }
      // binary comparison operators
      {
	std::valarray<bool> c(a > d);
	std::vector<bool> C(operator_less(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a >= d);
	std::vector<bool> C(operator_less_equal(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a < d);
	std::vector<bool> C(operator_greater(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a <= d);
	std::vector<bool> C(operator_greater_equal(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a == d);
	std::vector<bool> C(operator_equal_to(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a != d);
	std::vector<bool> C(operator_not_equal_to(d, A));
	assert(c == C);
      }
      // binary logical operators
      {
	std::valarray<bool> c(a && d);
	std::vector<bool> C(A && d);
	assert(c == C);
      }
      {
	std::valarray<bool> c(a || d);
	std::vector<bool> C(A || d);
	assert(c == C);
      }
      // binary logical operators
      {
	std::valarray<bool> c(a && d);
	std::vector<bool> C(d && A);
	assert(c == C);
      }
      {
	std::valarray<bool> c(a || d);
	std::vector<bool> C(d || A);
	assert(c == C);
      }
#if defined(_VALARRAY_NUMERIC_HPP_)
      // binary logical operators
      {
	std::valarray<bool> c(operator_logical_and(a, d));
	std::vector<bool> C(A && d);
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_logical_or(a, d));
	std::vector<bool> C(A || d);
	assert(c == C);
      }
      // binary logical operators
      {
	std::valarray<bool> c(operator_logical_and(d, a));
	std::vector<bool> C(d && A);
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_logical_or(d, a));
	std::vector<bool> C(d || A);
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_HPP_)      
      // binary comparison operators
      {
	std::valarray<bool> c(operator_less(a, d));
	std::vector<bool> C(operator_less(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_less_equal(a, d));
	std::vector<bool> C(operator_less_equal(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_greater(a, d));
	std::vector<bool> C(operator_greater(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_greater_equal(a, d));
	std::vector<bool> C(operator_greater_equal(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_equal_to(a, d));
	std::vector<bool> C(operator_equal_to(A, d));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_not_equal_to(a, d));
	std::vector<bool> C(operator_not_equal_to(A, d));
	assert(c == C);
      }
      // binary comparison operators
      {
	std::valarray<bool> c(operator_less(d, a));
	std::vector<bool> C(operator_less(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_less_equal(d, a));
	std::vector<bool> C(operator_less_equal(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_greater(d, a));
	std::vector<bool> C(operator_greater(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_greater_equal(d, a));
	std::vector<bool> C(operator_greater_equal(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_equal_to(d, a));
	std::vector<bool> C(operator_equal_to(d, A));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_not_equal_to(d, a));
	std::vector<bool> C(operator_not_equal_to(d, A));
	assert(c == C);
      }
#endif
      /*
	binary functions
      */
#if defined(_VALARRAY_NUMERIC_HPP_)
      {
	std::valarray<T> c(norm(a, d));
	std::vector<T> C(norm(A, d));
	assert(c == C);
      }
#endif
#if !0 // not T as real
#else
      {
        std::valarray<T> c(pow(a, d));
	std::vector<T> C(pow(A, d));
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_HPP_)
      {
	std::valarray<T> c(hypot(a, d));
	std::vector<T> C(hypot(A, d));
	assert(c == C);
      }
#endif
      {
	std::valarray<T> c(atan2(a, d));
	std::vector<T> C(atan2(A, d));
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_QUANTIFIED_OPERATORS_HPP_) && defined(_VECTOR_NUMERIC_QUANTIFIED_OPERATORS_HPP_)
      // binary forall operators
      {
	bool c(forall_operator_less(a, d));
	bool C(forall_operator_less(A, d));
	assert(c == C);
      }
      {
	bool c(forall_operator_less_equal(a, d));
	bool C(forall_operator_less_equal(A, d));
	assert(c == C);
      }
      {
	bool c(forall_operator_greater(a, d));
	bool C(forall_operator_greater(A, d));
	assert(c == C);
      }
      {
	bool c(forall_operator_greater_equal(a, d));
	bool C(forall_operator_greater_equal(A, d));
	assert(c == C);
      }
      {
	bool c(forall_operator_equal_to(a, d));
	bool C(forall_operator_equal_to(A, d));
	assert(c == C);
      }
      {
	bool c(forall_operator_not_equal_to(a, d));
	bool C(forall_operator_not_equal_to(A, d));
	assert(c == C);
      }
      // binary exists operators
      {
	bool c(exists_operator_less(a, d));
	bool C(exists_operator_less(A, d));
	assert(c == C);
      }
      {
	bool c(exists_operator_less_equal(a, d));
	bool C(exists_operator_less_equal(A, d));
	assert(c == C);
      }
      {
	bool c(exists_operator_greater(a, d));
	bool C(exists_operator_greater(A, d));
	assert(c == C);
      }
      {
	bool c(exists_operator_greater_equal(a, d));
	bool C(exists_operator_greater_equal(A, d));
	assert(c == C);
      }
      {
	bool c(exists_operator_equal_to(a, d));
	bool C(exists_operator_equal_to(A, d));
	assert(c == C);
      }
      {
	bool c(exists_operator_not_equal_to(a, d));
	bool C(exists_operator_not_equal_to(A, d));
	assert(c == C);
      }
      // binary forall operators
      {
	bool c(forall_operator_less(d, a));
	bool C(forall_operator_less(d, A));
	assert(c == C);
      }
      {
	bool c(forall_operator_less_equal(d, a));
	bool C(forall_operator_less_equal(d, A));
	assert(c == C);
      }
      {
	bool c(forall_operator_greater(d, a));
	bool C(forall_operator_greater(d, A));
	assert(c == C);
      }
      {
	bool c(forall_operator_greater_equal(d, a));
	bool C(forall_operator_greater_equal(d, A));
	assert(c == C);
      }
      {
	bool c(forall_operator_equal_to(d, a));
	bool C(forall_operator_equal_to(d, A));
	assert(c == C);
      }
      {
	bool c(forall_operator_not_equal_to(d, a));
	bool C(forall_operator_not_equal_to(d, A));
	assert(c == C);
      }
      // binary exists operators
      {
	bool c(exists_operator_less(d, a));
	bool C(exists_operator_less(d, A));
	assert(c == C);
      }
      {
	bool c(exists_operator_less_equal(d, a));
	bool C(exists_operator_less_equal(d, A));
	assert(c == C);
      }
      {
	bool c(exists_operator_greater(d, a));
	bool C(exists_operator_greater(d, A));
	assert(c == C);
      }
      {
	bool c(exists_operator_greater_equal(d, a));
	bool C(exists_operator_greater_equal(d, A));
	assert(c == C);
      }
      {
	bool c(exists_operator_equal_to(d, a));
	bool C(exists_operator_equal_to(d, A));
	assert(c == C);
      }
      {
	bool c(exists_operator_not_equal_to(d, a));
	bool C(exists_operator_not_equal_to(d, A));
	assert(c == C);
      }
#endif
    }
    for (size_t k=0; k<pow(m, n); ++k) {
      std::valarray<T> b(n);
      std::vector<T> B(n);
      size_t q = k;
      for (size_t i=0; i<n; ++i) {
	b[i] = (q % m);
	B[i] = (q % m);
	q /= m;
      }
      /*
	binary operators
      */
#if defined(_VECTOR_NUMERIC_HPP_)
      // binary compound assignment operators
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c += b;
	C += B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c -= b;
	C -= B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c *= b;
	C *= B;
	assert(c == C);
      }
      if (find(B.begin(), B.end(), T(0)) == B.end()) {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c /= b;
	C /= B;
	assert(c == C);
      }
#if !0 // not T as real
      if (find(B.begin(), B.end(), T(0)) == B.end()) {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c %= b;
	C %= B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c &= b;
	C &= B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c |= b;
	C |= B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c ^= b;
	C ^= B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c <<= b;
	C <<= B;
	assert(c == C);
      }
      {
	std::valarray<T> c(a);
	std::vector<T> C(A);
	c >>= b;
	C >>= B;
	assert(c == C);
      }
#endif
      // binary operators
      {
	std::valarray<T> c(a + b);
	std::vector<T> C(A + B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a - b);
	std::vector<T> C(A - B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a * b);
	std::vector<T> C(A * B);
	assert(c == C);
      }
      if (find(B.begin(), B.end(), T(0)) == B.end()) {
	std::valarray<T> c(a / b);
	std::vector<T> C(A / B);
	assert(c == C);
      }
#if !0 // not T as real
      if (find(B.begin(), B.end(), T(0)) == B.end()) {
	std::valarray<T> c(a % b);
	std::vector<T> C(A % B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a & b);
	std::vector<T> C(A & B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a | b);
	std::vector<T> C(A | B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a ^ b);
	std::vector<T> C(A ^ B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a << b);
	std::vector<T> C(A << B);
	assert(c == C);
      }
      {
	std::valarray<T> c(a >> b);
	std::vector<T> C(A >> B);
	assert(c == C);
      }
#endif
      // binary comparison operators
      {
	std::valarray<bool> c(a < b);
	std::vector<bool> C(operator_less(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a <= b);
	std::vector<bool> C(operator_less_equal(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a > b);
	std::vector<bool> C(operator_greater(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a >= b);
	std::vector<bool> C(operator_greater_equal(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a == b);
	std::vector<bool> C(operator_equal_to(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(a != b);
	std::vector<bool> C(operator_not_equal_to(A, B));
	assert(c == C);
      }
      // binary logical operators
      {
	std::valarray<bool> c(a && b);
	std::vector<bool> C(A && B);
	assert(c == C);
      }
      {
	std::valarray<bool> c(a || b);
	std::vector<bool> C(A || B);
	assert(c == C);
      }
#if defined(_VALARRAY_NUMERIC_HPP_)
      // binary logical operators
      {
	std::valarray<bool> c(operator_logical_and(a, b));
	std::vector<bool> C(A && B);
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_logical_or(a, b));
	std::vector<bool> C(A || B);
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_HPP_)
      // binary comparison operators
      {
	std::valarray<bool> c(operator_less(a, b));
	std::vector<bool> C(operator_less(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_less_equal(a, b));
	std::vector<bool> C(operator_less_equal(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_greater(a, b));
	std::vector<bool> C(operator_greater(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_greater_equal(a, b));
	std::vector<bool> C(operator_greater_equal(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_equal_to(a, b));
	std::vector<bool> C(operator_equal_to(A, B));
	assert(c == C);
      }
      {
	std::valarray<bool> c(operator_not_equal_to(a, b));
	std::vector<bool> C(operator_not_equal_to(A, B));
	assert(c == C);
      }
#endif
      /*
	binary functions
      */
#if !0 // not T as real
#else
      {
	std::valarray<T> c(pow(a, b));
	std::vector<T> C(pow(A, B));
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_HPP_)
      {
	std::valarray<T> c(hypot(a, b));
	std::vector<T> C(hypot(A, B));
	assert(c == C);
      }
#endif
      {
	std::valarray<T> c(atan2(a, b));
	std::vector<T> C(atan2(A, B));
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_QUANTIFIED_OPERATORS_HPP_) && defined(_VECTOR_NUMERIC_QUANTIFIED_OPERATORS_HPP_)
      // binary forall operators
      {
	bool c(forall_operator_less(a, b));
	bool C(forall_operator_less(A, B));
	assert(c == C);
      }
      {
	bool c(forall_operator_less_equal(a, b));
	bool C(forall_operator_less_equal(A, B));
	assert(c == C);
      }
      {
	bool c(forall_operator_greater(a, b));
	bool C(forall_operator_greater(A, B));
	assert(c == C);
      }
      {
	bool c(forall_operator_greater_equal(a, b));
	bool C(forall_operator_greater_equal(A, B));
	assert(c == C);
      }
      {
	bool c(forall_operator_equal_to(a, b));
	bool C(forall_operator_equal_to(A, B));
	assert(c == C);
      }
      {
	bool c(forall_operator_not_equal_to(a, b));
	bool C(forall_operator_not_equal_to(A, B));
	assert(c == C);
      }
      // binary exists operators
      {
	bool c(exists_operator_less(a, b));
	bool C(exists_operator_less(A, B));
	assert(c == C);
      }
      {
	bool c(exists_operator_less_equal(a, b));
	bool C(exists_operator_less_equal(A, B));
	assert(c == C);
      }
      {
	bool c(exists_operator_greater(a, b));
	bool C(exists_operator_greater(A, B));
	assert(c == C);
      }
      {
	bool c(exists_operator_greater_equal(a, b));
	bool C(exists_operator_greater_equal(A, B));
	assert(c == C);
      }
      {
	bool c(exists_operator_equal_to(a, b));
	bool C(exists_operator_equal_to(A, B));
	assert(c == C);
      }
      {
	bool c(exists_operator_not_equal_to(a, b));
	bool C(exists_operator_not_equal_to(A, B));
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_ORDERED_OPERATORS_HPP_)
      // binary ordered operators
      {
	bool c(ordered_operator_less(a, b));
	bool C(A < B);
	assert(c == C);
      }
      {
	bool c(ordered_operator_less_equal(a, b));
	bool C(A <= B);
	assert(c == C);
      }
      {
	bool c(ordered_operator_greater(a, b));
	bool C(A > B);
	assert(c == C);
      }
      {
	bool c(ordered_operator_greater_equal(a, b));
	bool C(A >= B);
	assert(c == C);
      }
      {
	bool c(ordered_operator_equal_to(a, b));
	bool C(A == B);
	assert(c == C);
      }
      {
	bool c(ordered_operator_not_equal_to(a, b));
	bool C(A != B);
	assert(c == C);
      }
#endif
#if defined(_VALARRAY_NUMERIC_ORDERED_OPERATORS_HPP_) && defined(_VECTOR_NUMERIC_ORDERED_OPERATORS_HPP_)
      // binary ordered operators
      {
	bool c(ordered_operator_less(a, b));
	bool C(ordered_operator_less(A, B));
	assert(c == C);
      }
      {
	bool c(ordered_operator_less_equal(a, b));
	bool C(ordered_operator_less_equal(A, B));
	assert(c == C);
      }
      {
	bool c(ordered_operator_greater(a, b));
	bool C(ordered_operator_greater(A, B));
	assert(c == C);
      }
      {
	bool c(ordered_operator_greater_equal(a, b));
	bool C(ordered_operator_greater_equal(A, B));
	assert(c == C);
      }
      {
	bool c(ordered_operator_equal_to(a, b));
	bool C(ordered_operator_equal_to(A, B));
	assert(c == C);
      }
      {
	bool c(ordered_operator_not_equal_to(a, b));
	bool C(ordered_operator_not_equal_to(A, B));
	assert(c == C);
      }
#endif
#if defined(_CONTAINER_NUMERIC_HPP_)
      {
	std::valarray<T> c(::min(a, b));
	std::vector<T> C(::min(A, B));
	assert(c == C);
      }
      {
	std::valarray<T> c(::max(a, b));
	std::vector<T> C(::max(A, B));
	assert(c == C);
      }
      {
	std::valarray<T> c(positive_min(a, b));
	std::vector<T> C(positive_min(A, B));
	assert(c == C);
      }
      {
	std::valarray<T> c(negative_max(a, b));
	std::vector<T> C(negative_max(A, B));
	assert(c == C);
      }
#endif
      for (T d(0); d<T(m); ++d) {
	/*
	  ternary operators
	*/
#if defined(_VECTOR_NUMERIC_HPP_)
#if defined(_VALARRAY_NUMERIC_HPP_)
	{
	  std::valarray<T> c(operator_conditional(a, b, d));
	  std::vector<T> C(operator_conditional(A, B, d));
	  assert(c == C);
	}
	{
	  std::valarray<T> c(operator_conditional(d, a, b));
	  std::vector<T> C(operator_conditional(d, A, B));
	  assert(c == C);
        }
#endif
#endif
      }
      for (size_t k=0; k<pow(m, n); ++k) {
	std::valarray<T> c(n);
	std::vector<T> C(n);
	size_t q = k;
	for (size_t i=0; i<n; ++i) {
	  c[i] = (q % m);
	  C[i] = (q % m);
	  q /= m;
	}
	/*
	  ternary operators
	*/
#if defined(_VECTOR_NUMERIC_HPP_)
#if defined(_VALARRAY_NUMERIC_HPP_)
	{
	  std::valarray<T> d(operator_conditional(a, b, c));
	  std::vector<T> D(operator_conditional(A, B, C));
	  assert(d == D);
	}
#endif
#endif
      }
    }
  }
  return 0;
}
