/*
  vector_numeric.hpp

  Copyright (C) 2011-2016 Taiji Yamada <taiji@aihara.co.jp>
 */
#ifndef _VECTOR_NUMERIC_HPP_
#define _VECTOR_NUMERIC_HPP_
#include <vector>
#include "container_numeric.hpp"
#include <boost/multi_array.hpp>

template <typename T>
inline
std::vector<T> operator+(const std::vector<T> &a)
{
  return std::vector<T>(a);
}
template <typename T>
inline
std::vector<T> operator-(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = -*it;
  return c;
}
template <typename T>
inline
std::vector<T> operator~(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = ~*it;
  return c;
}
template <typename T>
inline
std::vector<bool> operator!(const std::vector<T> &a)
{
  std::vector<bool> c(a.size());
  typename std::vector<bool>::iterator it;
  typename std::vector<T>::const_iterator jt;
  for (it=c.begin(), jt=a.begin(); it!=c.end(); ++it, ++jt)
    *it = !*jt;
  return c;
}

template <typename T, typename U>
inline
std::vector<T> operator+(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it + b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator+(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] + b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator+(const U a, const std::vector<T> b)
{
  std::vector<T> c(b);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = a + *it;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator+=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it += b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator+=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] += b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator-(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it - b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator-(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] - b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator-(const U a, const std::vector<T> b)
{
  std::vector<T> c(b);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = a - *it;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator-=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it -= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator-=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] -= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator*(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it * b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator*(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] * b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator*(const U a, const std::vector<T> b)
{
  std::vector<T> c(b);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = a * *it;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator*=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it *= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator*=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] *= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator/(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it / b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator/(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] / b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator/(const U a, const std::vector<T> b)
{
  std::vector<T> c(b);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = a / *it;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator/=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it /= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator/=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] /= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator%(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it % b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator%(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] % b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator%(const U a, const std::vector<T> &b)
{
  std::vector<T> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a % b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator%=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it %= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator%=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] %= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator&(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it & b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator&(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] & b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator&(const U a, const std::vector<T> &b)
{
  std::vector<T> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a & b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator&=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it &= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator&=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] &= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator|(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it | b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator|(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] | b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator|(const U a, const std::vector<T> &b)
{
  std::vector<T> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a | b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator|=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it |= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator|=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] |= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator^(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it ^ b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator^(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] ^ b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator^(const U a, const std::vector<T> &b)
{
  std::vector<T> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a ^ b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator^=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it ^= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator^=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] ^= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator<<(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it << b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator<<(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] << b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator<<(const U a, const std::vector<T> &b)
{
  std::vector<T> c(b);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = a << *it;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator<<=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it <<= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator<<=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] <<= b[i];
  return a;
}

template <typename T, typename U>
inline
std::vector<T> operator>>(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = *it >> b;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator>>(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] >> b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator>>(const U a, const std::vector<T> &b)
{
  std::vector<T> c(b);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = a >> *it;
  return c;
}
template <typename T, typename U>
inline
std::vector<T> operator>>=(std::vector<T> &a, const U b)
{
  for (typename std::vector<T>::iterator it=a.begin(); it!=a.end(); ++it)
    *it >>= b;
  return a;
}
template <typename T, typename U>
inline
std::vector<T> operator>>=(std::vector<T> &a, const std::vector<U> &b)
{
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    a[i] >>= b[i];
  return a;
}

#if 0 /* ambiguous overload at bits/stl_vector.h */
template <typename T, typename U>
inline
std::vector<bool> operator<(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] < b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator<(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] < b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator<(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a < b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator<=(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] <= b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator<=(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] <= b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator<=(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a <= b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator>(const std::vector<T> &a, const U b)
{
  return b < a;
}
template <typename T, typename U>
inline
std::vector<bool> operator>(const std::vector<T> &a, const std::vector<U> &b)
{
  return b < a;
}
template <typename T, typename U>
inline
std::vector<bool> operator>(const U a, const std::vector<T> &b)
{
  return b < a;
}
template <typename T, typename U>
inline
std::vector<bool> operator>=(const std::vector<T> &a, const U b)
{
  return b <= a;
}
template <typename T, typename U>
inline
std::vector<bool> operator>=(const std::vector<T> &a, const std::vector<U> &b)
{
  return b <= a;
}
template <typename T, typename U>
inline
std::vector<bool> operator>=(const U a, const std::vector<T> &b)
{
  return b <= a;
}

template <typename T, typename U>
inline
std::vector<bool> operator==(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] == b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator==(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] == b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator==(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a == b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator!=(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] != b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator!=(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] != b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator!=(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a != b[i];
  return c;
}
#else
template <typename T, typename U>
inline
std::vector<bool> operator_less(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] < b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_less(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] < b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_less(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a < b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator_less_equal(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] <= b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_less_equal(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] <= b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_less_equal(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a <= b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator_greater(const std::vector<T> &a, const U b)
{
  return operator_less(b, a);
}
template <typename T, typename U>
inline
std::vector<bool> operator_greater(const std::vector<T> &a, const std::vector<U> &b)
{
  return operator_less(b, a);
}
template <typename T, typename U>
inline
std::vector<bool> operator_greater(const U a, const std::vector<T> &b)
{
  return operator_less(b, a);
}
template <typename T, typename U>
inline
std::vector<bool> operator_greater_equal(const std::vector<T> &a, const U b)
{
  return operator_less_equal(b, a);
}
template <typename T, typename U>
inline
std::vector<bool> operator_greater_equal(const std::vector<T> &a, const std::vector<U> &b)
{
  return operator_less_equal(b, a);
}
template <typename T, typename U>
inline
std::vector<bool> operator_greater_equal(const U a, const std::vector<T> &b)
{
  return operator_less_equal(b, a);
}

template <typename T, typename U>
inline
std::vector<bool> operator_equal_to(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] == b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_equal_to(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] == b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_equal_to(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a == b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator_not_equal_to(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] != b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_not_equal_to(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] != b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_not_equal_to(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a != b[i];
  return c;
}
#endif

#if !0 /* not ambiguous overload at bits/stl_vector.h */
template <typename T, typename U>
inline
std::vector<bool> operator&&(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] && b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator&&(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] && b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator&&(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a && b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator||(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] || b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator||(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] || b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator||(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a || b[i];
  return c;
}
#else
template <typename T, typename U>
inline
std::vector<bool> operator_logical_and(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] && b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_logical_and(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] && b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_logical_and(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a && b[i];
  return c;
}

template <typename T, typename U>
inline
std::vector<bool> operator_logical_or(const std::vector<T> &a, const U b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] || b;
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_logical_or(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<bool> c(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    c[i] = a[i] || b[i];
  return c;
}
template <typename T, typename U>
inline
std::vector<bool> operator_logical_or(const U a, const std::vector<T> &b)
{
  std::vector<bool> c(b.size());
  for (typename std::vector<T>::size_type i=0; i<b.size(); ++i)
    c[i] = a || b[i];
  return c;
}
#endif

template <typename T, typename U>
inline
const std::vector<T> &operator_conditional(const U a, const std::vector<T> &b, const std::vector<T> &c)
{
  return a ? b : c;
}
template <typename T, typename U>
inline
std::vector<T> operator_conditional(const std::vector<U> &a, const std::vector<T> &b, const std::vector<T> &c)
{
  std::vector<T> d(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    d[i] = a[i] ? b[i] : c[i];
  return d;
}
template <typename T, typename U, typename V>
inline
std::vector<T> operator_conditional(const std::vector<U> &a, const std::vector<T> &b, const V c)
{
  std::vector<T> d(a.size());
  for (typename std::vector<T>::size_type i=0; i<a.size(); ++i)
    d[i] = a[i] ? b[i] : c;
  return d;
}

template <typename T>
struct vector_index_less {
  vector_index_less(const std::vector<T> &p) : v(p) {}
  bool operator()(const typename std::vector<T>::size_type a, const typename std::vector<T>::size_type b)
  {
    return v[a] < v[b];
  }
private:
  const std::vector<T> &v;
};

template <typename T>
struct vector_index_greater {
  vector_index_greater(const std::vector<T> &p) : v(p) {}
  bool operator()(const typename std::vector<T>::size_type a, const typename std::vector<T>::size_type b)
  {
    return v[a] > v[b];
  }
private:
  const std::vector<T> &v;
};

template <typename T>
inline
T norm2(const std::vector<T> &a)
{
  T c(0);
  for (typename std::vector<T>::const_iterator it=a.begin(); it!=a.end(); ++it)
    c += *it * *it;
  return sqrt(c);
}

template <typename T, typename U>
inline
T norm(const std::vector<T> &a, const U b = U(2))
{
  T c(0);
  if (b != U(0)) {
    for (typename std::vector<T>::const_iterator it=a.begin(); it!=a.end(); ++it)
      c += pow(std::abs(*it), b);
    c = pow(c, U(1)/b);
  }
  else { // isinf(b)
    typename std::vector<T>::const_iterator it(a.begin());
    c = std::abs(*it++);
    for (; it!=a.end(); ++it)
      c = std::max(std::abs(*it), c);
  }
  return c;
}

template <typename T>
inline
T prod(const std::vector<T> &a)
{
  T c(1);
  for (typename std::vector<T>::const_iterator it=a.begin(); it!=a.end(); ++it)
    c *= *it;
  return c;
}

template <typename T>
inline
std::vector<T> abs(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = abs(*it);
  return c;
}
template <typename T, typename U>
inline
std::vector<T> pow(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = pow(*it, b);
  return c;
}
template <typename T, typename U>
inline
std::vector<T> pow(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a);
  typename std::vector<T>::iterator it;
  typename std::vector<U>::const_iterator jt;
  for (it=c.begin(), jt=b.begin(); it!=c.end(); ++it, ++jt)
    *it = pow(*it, *jt);
  return c;
}
template <typename T>
inline
std::vector<T> sqrt(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = sqrt(*it);
  return c;
}
template <typename T>
inline
std::vector<T> cbrt(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = cbrt(*it);
  return c;
}
template <typename T, typename U>
inline
std::vector<T> hypot(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = hypot(*it, b);
  return c;
}
template <typename T, typename U>
inline
std::vector<T> hypot(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a);
  typename std::vector<T>::iterator it;
  typename std::vector<U>::const_iterator jt;
  for (it=c.begin(), jt=b.begin(); it!=c.end(); ++it, ++jt)
    *it = hypot(*it, *jt);
  return c;
}
template <typename T>
inline
std::vector<T> exp(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = exp(*it);
  return c;
}
template <typename T>
inline
std::vector<T> exp2(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = exp2(*it);
  return c;
}
template <typename T>
inline
std::vector<T> expm1(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = expm1(*it);
  return c;
}
template <typename T>
inline
std::vector<T> log(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = log(*it);
  return c;
}
template <typename T>
inline
std::vector<T> log10(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = log10(*it);
  return c;
}
template <typename T>
inline
std::vector<T> log2(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = log2(*it);
  return c;
}
template <typename T>
inline
std::vector<T> log1p(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = log1p(*it);
  return c;
}
template <typename T>
inline
std::vector<T> sin(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = sin(*it);
  return c;
}
template <typename T>
inline
std::vector<T> cos(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = cos(*it);
  return c;
}
template <typename T>
inline
std::vector<T> tan(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = tan(*it);
  return c;
}
template <typename T>
inline
std::vector<T> sinh(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = sinh(*it);
  return c;
}
template <typename T>
inline
std::vector<T> cosh(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = cosh(*it);
  return c;
}
template <typename T>
inline
std::vector<T> tanh(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = tanh(*it);
  return c;
}
template <typename T>
inline
std::vector<T> asin(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = asin(*it);
  return c;
}
template <typename T>
inline
std::vector<T> acos(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = acos(*it);
  return c;
}
template <typename T>
inline
std::vector<T> atan(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = atan(*it);
  return c;
}
template <typename T, typename U>
inline
std::vector<T> atan2(const std::vector<T> &a, const U b)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = atan2(*it, b);
  return c;
}
template <typename T, typename U>
inline
std::vector<T> atan2(const std::vector<T> &a, const std::vector<U> &b)
{
  std::vector<T> c(a);
  typename std::vector<T>::iterator it;
  typename std::vector<U>::const_iterator jt;
  for (it=c.begin(), jt=b.begin(); it!=c.end(); ++it, ++jt)
    *it = atan2(*it, *jt);
  return c;
}
template <typename T>
inline
std::vector<T> asinh(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = asinh(*it);
  return c;
}
template <typename T>
inline
std::vector<T> acosh(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = acosh(*it);
  return c;
}
template <typename T>
inline
std::vector<T> atanh(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = atanh(*it);
  return c;
}
template <typename T>
inline
std::vector<T> erf(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = erf(*it);
  return c;
}
template <typename T>
inline
std::vector<T> erfc(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = erfc(*it);
  return c;
}
template <typename T>
inline
std::vector<T> tgamma(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = tgamma(*it);
  return c;
}
template <typename T>
inline
std::vector<T> lgamma(const std::vector<T> &a)
{
  std::vector<T> c(a);
  for (typename std::vector<T>::iterator it=c.begin(); it!=c.end(); ++it)
    *it = lgamma(*it);
  return c;
}


template <typename T>
inline
std::vector<T> make_vector(const boost::detail::multi_array::sub_array<T, 1> &a)
{
  std::vector<T> v(a.size());
  for (std::size_t i=0; i<a.size(); ++i)
    v[i] = a[i];
  return v;
}


template <typename T>
inline
boost::multi_array<T, 2> identity_matrix(const size_t M, const size_t N)
{
  boost::multi_array<T, 2> I(boost::extents[M][N]);
  for (size_t i=0; i<M; ++i) {
    std::fill(I[i].begin(), I[i].end(), T(0));
    if (i < N)
      I[i][i] = T(1);
  }
  return I;
}
template <typename T>
inline
boost::multi_array<T, 2> matrix(const size_t M, const size_t N)
{
  boost::multi_array<T, 2> m(boost::extents[M][N]);
  for (size_t i=0; i<M; ++i)
    std::fill(m[i].begin(), m[i].end(), T(0));
  return m;
}
template <typename T>
inline
boost::multi_array<T, 2> matrix(const boost::multi_array<T, 2> &a)
{
  boost::multi_array<T, 2> m(boost::extents[a.size()][a[0].size()]);
  for (size_t i=0; i<a.size(); ++i)
    std::copy(a[i].begin(), a[i].end(), m[i].begin());
  return m;
}
template <typename T>
inline
std::vector<T> prod(const boost::multi_array<T, 2> &a, const std::vector<T> &b)
{
  std::vector<T> c(b.size());
  for (size_t i=0, j; i<b.size(); ++i)
    for (c[i] = 0, j=0; j<b.size(); ++j)
      c[i] += a[i][j]*b[j];
  return c;
}

#endif
